with Ada.Command_Line;
with Ada.Text_IO;
with Ada.Unchecked_Deallocation;
with Scream.Build;
with Scream.Output.HTML;
procedure Scream.Command is
	procedure Log (S : in String) is
	begin
		Ada.Text_IO.Put_Line (Ada.Text_IO.Standard_Error.all, S);
	end Log;
	Command_Line_Error : exception;
	type String_Access is access String;
	procedure Free is new Ada.Unchecked_Deallocation (String, String_Access);
	type Format_Access is access Output.Format'Class;
	procedure Free is
		new Ada.Unchecked_Deallocation (Output.Format'Class, Format_Access);
	Format : Format_Access := new Output.HTML.HTML;
	Force : Boolean := False;
	Verbose : Boolean := False;
	Help : Boolean := False;
	Base_Directory : String_Access := new String'(".");
	Error : Boolean;
begin
	Scream.Log := Log'Unrestricted_Access;
	for I in 1 .. Ada.Command_Line.Argument_Count loop
		declare
			Arg : constant String := Ada.Command_Line.Argument (I);
		begin
			if Arg = "-f" or else Arg = "--force" then
				Force := True;
			elsif Arg = "-h" or else Arg = "--help" then
				Help := True;
			elsif Arg = "--html" then
				Free (Format);
				Format := new Output.HTML.HTML;
			elsif Arg = "-v" or else Arg = "--verbose" then
				Verbose := True;
			elsif Arg (Arg'First) = '-' then
				raise Command_Line_Error with "Invalid command-line argument.";
			else
				Free (Base_Directory);
				Base_Directory := new String'(Arg);
			end if;
		end;
	end loop;
	if Help then
		declare
			use Ada.Text_IO;
		begin
			Put_Line ("usage: nightscream PATH");
			Put_Line ("  -f --force   force to remake all files.");
			Put_Line ("  -h --help    print this message.");
			Put_Line ("     --html    output HTML (default).");
			Put_Line ("  -v --verbose print more information.");
		end;
	else
		Build (
			Format => Format,
			Base_Directory => Base_Directory.all,
			Force => Force,
			Verbose => Verbose,
			Error => Error);
	end if;
	Free (Base_Directory);
	Free (Format);
	if Error then
		Ada.Command_Line.Set_Exit_Status (Ada.Command_Line.Failure);
	end if;
end Scream.Command;
