package body Scream.Markup.Emphasis is
	
	procedure Parse_Argument (
		Parsing : not null access Markup.Parsing;
		Mark_Style : Style;
		Argument : String;
		Argument_Position : Source_Position;
		Result : out Style_Set)
	is
		I : Source_Position := Argument_Position;
		First : Source_Position;
		Last : Natural;
		S : Style;
	begin
		for I in Result'Range loop
			Result (I) := 0;
		end loop;
		Result (Mark_Style) := 1;
		while I.Index <= Argument'Last loop
			First := I;
			Take_Word (Argument, Last, I);
			begin
				S := Style'Value (Argument (First.Index .. Last));
				Result (S) := Result (S) + 1;
			exception
				when Constraint_Error =>
					Error (
						Parsing,
						First,
						Argument (First.Index .. Last) & " is undefined emphasis function.");
			end;
			Skip_Spaces (Argument, I);
			exit when I.Index > Argument'Last or else Argument (I.Index) /= ':';
			Next (Argument, I);
			Skip_Spaces (Argument, I);
		end loop;
	end Parse_Argument;
	
	type Block_Access is access Block;
	type Inline_Access is access Inline;
	
	overriding procedure Iterate (
		Item : in Inline;
		Process : not null access procedure (Item : in Element'Class)) is
	begin
		Iterate (Item.Items, Process);
	end Iterate;
	
	overriding procedure Parse_Block (
		The_Mark : in Mark_Type;
		Argument : in String;
		Argument_Position : in Source_Position;
		Parsing : not null access Markup.Parsing;
		Previous : in out Block_Access_Access;
		Source : in String;
		Position : in out Source_Position;
		Stopper : not null access function (Source : String) return Boolean)
	is
		function Is_Closed is new Generic_Inline_Stopper (Stopper.all);
		New_Item : not null Block_Access :=
			new Block'(Markup.Block with Style => (others => 0), Items => null);
	begin
		Parse_Argument (
			Parsing,
			The_Mark.Style,
			Argument,
			Argument_Position,
			New_Item.Style);
		Append (Parsing, Previous, New_Item);
		Parse_Inline (
			Parsing,
			New_Item.Items'Unchecked_Access,
			Source,
			Position,
			New_Item.Contents,
			Is_Closed'Access);
	end Parse_Block;
	
	overriding procedure Parse_Inline (
		The_Mark : in Mark_Type;
		Argument : in String;
		Argument_Position : in Source_Position;
		Parsing : not null access Markup.Parsing;
		Previous : in out Inline_Access_Access;
		Source : in String;
		Position : in out Source_Position;
		Stopper : not null access function (Source : String) return Boolean)
	is
		New_Item : not null Inline_Access :=
			new Inline'(Markup.Inline with Style => (others => 0), Items => null);
	begin
		Parse_Argument (
			Parsing,
			The_Mark.Style,
			Argument,
			Argument_Position,
			New_Item.Style);
		Append (Parsing, Previous, New_Item);
		Parse_Inline (
			Parsing,
			New_Item.Items'Unchecked_Access,
			Source,
			Position,
			New_Item.Contents,
			Stopper);
	end Parse_Inline;
	
	overriding function Create (Params : not null access Params_Type)
		return Mark_Type is
	begin
		return (Style => Style'Val (Params.Data));
	end Create;
	
begin
	for I in Style loop
		Register (Style'Image (I), Mark_Type'Tag, (Data => Style'Pos (I)));
	end loop;
end Scream.Markup.Emphasis;
