package body Scream.Markup.List is
	
	type Block_Access is access Block;
	
	overriding procedure Iterate (
		Item : in Block;
		Process : not null access procedure (Item : in Element'Class))
	is
		I : Lists.Cursor := Item.Items.First;
	begin
		while Lists.Has_Element (I) loop
			Iterate (Item.Items.Constant_Reference (I).Items, Process);
			Lists.Next (I);
		end loop;
	end Iterate;
	
	overriding procedure Parse_Block (
		The_Mark : in Mark;
		Argument : in String;
		Argument_Position : in Source_Position;
		Parsing : not null access Markup.Parsing;
		Previous : in out Block_Access_Access;
		Source : in String;
		Position : in out Source_Position;
		Stopper : not null access function (Source : String) return Boolean)
	is
		New_Item : not null Block_Access :=
			new Block'(
				Markup.Block with Style => Unordered, Items => Lists.Empty_List);
		Min_Level : Natural;
	begin
		if Argument /= "" then
			New_Item.Style := Style'Value (Argument);
		end if;
		Append (Parsing, Previous, New_Item);
		Append (Parsing, New_Item.Contents, "");
		while not Stopper (Source (Position.Index .. Source'Last)) loop
			declare
				Item : List.Item;
				Additional_Contents : Range_Type;
				Variable_Previous : Inline_Access_Access := Item.Items'Unchecked_Access;
			begin
				Take_Same_Symbols (Source, '-', Item.Level, Position);
				Skip_Spaces (Source, Position);
				Parse_Inline (
					Parsing,
					Variable_Previous,
					Source (Position.Index .. Source'Last),
					Position,
					Item.Contents,
					Is_End_Of_Line'Access);
				Skip_Single_Line_Feed (Source, Position);
				while Position.Index < Source'Last
					and then (
						Source (Position.Index) = ' '
						or else Source (Position.Index) = Ascii.HT)
				loop
					Skip_Spaces (Source, Position);
					Append (Parsing, Additional_Contents, (1 => Ascii.LF));
					while Variable_Previous.all /= null loop
						Variable_Previous := Variable_Previous.all.Next'Access;
					end loop;
					Parse_Inline (
						Parsing,
						Variable_Previous,
						Source (Position.Index .. Source'Last),
						Position,
						Additional_Contents,
						Is_End_Of_Line'Access);
					Skip_Single_Line_Feed (Source, Position);
					Item.Contents.Last := Additional_Contents.Last;
				end loop;
				New_Item.Contents.Last := Item.Contents.Last;
				Lists.Append (New_Item.Items, Item);
			end;
		end loop;
		Min_Level := Natural'Last;
		declare
			I : Lists.Cursor := New_Item.Items.First;
		begin
			while Lists.Has_Element (I) loop
				Min_Level := Natural'Min (
					Min_Level,
					New_Item.Items.Constant_Reference (I).Level);
				Lists.Next (I);
			end loop;
		end;
		declare
			I : Lists.Cursor := New_Item.Items.First;
		begin
			while Lists.Has_Element (I) loop
				declare
					Subitem : List.Item renames New_Item.Items.Reference (I);
				begin
					Subitem.Level := Subitem.Level - Min_Level;
				end;
				Lists.Next (I);
			end loop;
		end;
	end Parse_Block;
	
	overriding function Create (Params : not null access Params_Type)
		return Mark is
	begin
		return (null record);
	end Create;
	
begin
	Register ("LIST", Mark'Tag);
end Scream.Markup.List;
