with Ada.Calendar;
with Ada.Containers.Indefinite_Ordered_Sets;
with Ada.Unchecked_Deallocation;
package Scream.Pages is
	
	-- info file name:
	--   .scream/NAME.dat
	-- backup file name:
	--   .scream/YYYYMMDDhhmmss NAME.gz
	
	Source_Extension : constant String := "scream.txt";
	Info_Extension : constant String := "dat";
	Backup_Extension : constant String := "gz";
	Storage_Directory : constant String := ".scream";
	Index : constant String := "index";
	
	package Page_Sets is new Ada.Containers.Indefinite_Ordered_Sets (String);
	
	type Page_Info is limited record
		Creation_Time : Ada.Calendar.Time;
		Build_Time : Ada.Calendar.Time;
		Reference : aliased Page_Sets.Set;
		Is_New : Boolean;
	end record;
	
	function Get_Page_Info (Path : String) return Page_Info;
	procedure Save_Page_Info (
		Path : String;
		Info : Page_Info;
		Source : String;
		Modified : Boolean);
	
	type Page_Info_Access is access Page_Info;
	
	procedure Free is new Ada.Unchecked_Deallocation (Page_Info, Page_Info_Access);
	
	function Reference_Path (Path : String) return String;
	
	-- path operations
	
	function Containing_Directory (Name : String) return String;
	
	function Compose (Containing_Directory : String; Name : String)
		return String;
	
	function Relative_Name (
		Base_Directory : String;
		Name : String;
		Drive_Support : Boolean := False)
		return String;
	
end Scream.Pages;
