with Ada.Strings.Unbounded;
procedure Scream.Parse (
	File_Name : in String;
	Source : in String;
	Page : out Markup.Page;
	Reference : not null access Pages.Page_Sets.Set;
	Errors : out Markup.Error_Lists.List)
is
	Parsing : aliased Markup.Parsing := (
		Text => Page.Text'Unchecked_Access,
		All_Elements => Page.All_Elements'Unchecked_Access,
		Reference => Reference,
		Errors => Markup.Error_Lists.Empty_List);
	Position : Markup.Source_Position := (
		File_Name => Ada.Strings.Unbounded.To_Unbounded_String (File_Name),
		Index => Source'First,
		Line => 1,
		Column => 1);
	Title_Last : Natural;
	Dummy_Contents : Markup.Range_Type;
begin
	Pages.Page_Sets.Clear (Reference.all);
	Page.Text := Ada.Strings.Unbounded.Null_Unbounded_String;
	Markup.Take_Line (Source, Title_Last, Position);
	Markup.Append (
		Parsing'Access,
		Page.Title,
		Source (Source'First .. Title_Last));
	Page.Items := null;
	Page.All_Elements := null;
	Markup.Parse_Block (
		Parsing'Access,
		Page.Items'Unchecked_Access,
		Source (Position.Index .. Source'Last),
		Position,
		Dummy_Contents,
		Markup.Is_Empty'Access);
	Errors := Parsing.Errors;
end Scream.Parse;
